/*
 * DataClass.h
 *
 *  Created on: Mar 19, 2010
 *      Author: roger
 */

#ifndef DATACLASS_H_
#define DATACLASS_H_

#include <string>
#include <vector>
#include <iostream>
#include <fstream>
#include <termios.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
#include <boost/date_time/posix_time/posix_time.hpp>

#include "parameters.hpp"
#include "globalVar.h"


using namespace std;


// Class for reading & storing data
class DataClass {
public:
	DataClass(string aFile);
	virtual ~DataClass();

public:
        int nb_c,nb_l;                          // Matrix dimensions
        string inputfile;                       // serial Port where data is read
        vector<vector<vector<double> > > data;  // vector of matrixes where data is stored
        int fd;                                 // reference for file/port link
        int *matrixFilter;                      // matrix filter filled by 'getMaxFilter'
        vector<int> timedata;                   // vector of time values
        boost::posix_time::ptime startTIME;     // gets start time of program
        ofstream savefile;                      // save file for un-pre-processed mat data
        double * rawMat;                        // saves raw Mat for display
        unsigned char * null_buf;                        // variable buffer with only zero elements



public:
        void recenter_transmission();           // makes sure data chains are in the right position (ie start at PAD0 = 10)
        void removeMatrix();                    // removes a matrix vector
        void readMatrix();                      // reads a matrix from Port
        double getMat(int i,int j);             // returns data(i,j) of the oldest matrix
        int getNbMat();                         // returns the number of matrices currently in memory
        int normalizeData(unsigned char s_buf); // normalises the data read from the port
        int normalizeData(unsigned char s_buf,int index);
        void Close();                           // closes communication
        void getMaxFilter(unsigned char* s_buf);// fills in a matrix filter to reduce noise
        int getTime();                          // returns oldest data time
        void initTime();                        // initializes startTime
        void saveINfile(unsigned char* s_buf,int time);  // saves char data sequence in file
        int getINTdata(unsigned char s_buf);   // returns an normalized int to be written in file
        void saveINvar(unsigned char* s_buf);   // saves raw mat in var for display
};

#endif /* DATACLASS_H_ */
