/*
 * DataFILEClass.cpp
 *
 *  Created on: Mar 19, 2010
 *      Author: roger
 */

#include "DataFILEClass.h"

void DataFILEClass::readHeader() {
    // Opening File stream
    ifstream inputFile(inputfile.c_str());
    if (!inputFile.is_open()) // if unable to open file
    {
        std::cout << "Error opening data file !" << std::endl;
        return;
    }

    boost::posix_time::ptime startTIME = boost::posix_time::ptime(boost::posix_time::microsec_clock::universal_time());
    inputFile >> startTIME;

    inputFile.close();
}

DataFILEClass::DataFILEClass(string aFile) {
    inputfile = aFile;
    index_counter = -1;
    nb_l = NBL;
    nb_c = NBC;
    end_file = false;
    matrixFilter = new int[nb_l*nb_c];

}

DataFILEClass::~DataFILEClass() {
    delete[] matrixFilter;
    for(int i=0;i<data.size();i++)
    {
        delete data[i];
    }
    data.clear();
}

void DataFILEClass::removeMatrix()
{
    data.erase(data.begin());
    timedata.erase(timedata.begin());
}

void DataFILEClass::readMatrix(int mat_index)
{
    // Opening File stream
    ifstream inputFile(inputfile.c_str());
    if (!inputFile.is_open())		// if unable to open file
    {
        cout << "Error opening data file !" << endl;
        return;
    }

    double buf;

    if(!useFILEfilter)  // if file is of raw matrices (then it has no header)
    {
        // ditching the header
        boost::posix_time::ptime startTIME = boost::posix_time::ptime(boost::posix_time::microsec_clock::universal_time());
        inputFile >> startTIME;		// date/time of simulation
    }

    int t_currentRelativeTIME;
    int aspeed=0;
    for(int i=0;i<mat_index;i++)			// ditching (not reading) first part of file
    {
        for(int i1=0;i1<nb_l;i1++)
        {
            for(int i2=0;i2<nb_c;i2++)
            {
                inputFile >> buf;
            }
        }
        inputFile >> t_currentRelativeTIME;
        if(readSpeed==1)
            inputFile >> aspeed;
    }

    if(inputFile.eof())			// if not end of file
    {
        end_file = true;
        cout << "!!!!!!!!!!end of data source file!!!!!!!!!!!!" << endl;
        return;
    }

    int *tmp_tab = new int[nb_l*nb_c];
    double *tab = new double[nb_l*nb_c];
    for(int w=0;w<nb_l*nb_c;w++)
    {
        inputFile >> buf;
        if(inputFile.eof())			// if not end of file
        {
            end_file = true;
            cout << "!!!!!!!!!!end of data source file!!!!!!!!!!!!" << endl;
            return;
        }
        tmp_tab[w]=buf;
    }
    inputFile >> t_currentRelativeTIME;
    getMaxFilter(tmp_tab);

    // filling of data with desired matrix
    int counter = 0;
    while(counter<nb_l*nb_c && !end_file)
    {
        if(useFILEfilter == 1)
        {
            buf = normalizeData(tmp_tab[counter],counter);
        }
        else
        {
            buf = tmp_tab[counter];
        }
        tab[counter] = buf;
        counter ++;
    }
    inputFile.close();			// closing file stream
    index_counter = mat_index;
    data.push_back(tab);
    timedata.push_back(t_currentRelativeTIME);
    delete[] tmp_tab;
}

double DataFILEClass::getMat(int i,int j)
{
    //	return data[0][i][j];
    return data[0][i*nb_c+j];
}

void DataFILEClass::readNextMatrix()
{
    readMatrix(index_counter + 1);
}

int DataFILEClass::getNbMat()
{
    return data.size();
}

bool DataFILEClass::checkIndex()
{
    return !end_file;
}

/* applies defined filter to data matrix */
int DataFILEClass::normalizeData(int s_buf,int index)
{
    int tmp = s_buf;
    int atemp = PAD_sleep_state*static_threshold/100;    // 5% of normal state of macanoreceptor

    if (matrixFilter[index]!=0 && tmp > atemp)
        return tmp;
    else
        return 0;
}

/* builds matrix filter depending on values aqcuired by port */
void DataFILEClass::getMaxFilter(int *s_buf)
{
    /* extracts max value mesured */
    int max=0;
    float d_threshold = dynamic_threshold /100;
    int t_max;
    for(int i=0;i<nb_l*nb_c;i++)
    {
        matrixFilter[i] = 0;                                  // resets old version of filter to 0
        if(max<s_buf[i])
            max = s_buf[i];
    }
    if(max > PAD_sleep_state*static_threshold/100)              // if data has been acquired
    {
        for(int i=0;i<nb_l*nb_c;i++)                          // finding mecanoreceptors with values > max/2
        {
            if(max*d_threshold<s_buf[i])  // if value > max/2
            {
                t_max = s_buf[i];
                matrixFilter[i] = 2;
                for(int k=nb_c-1;k<=nb_c+1;k++)                 // looking at the values of the 8 mecanoreceptors around it
                {
                    if(i-k>=0 && t_max*d_threshold<s_buf[i-k])
                    {
                        matrixFilter[i-k]=1;
                    }
                    if(i+k<nb_c*nb_l && t_max*d_threshold<s_buf[i+k])
                    {
                        matrixFilter[i+k]=1;
                    }
                }
                if(i-1>=0 && t_max*d_threshold<s_buf[i-1])
                {
                    matrixFilter[i-1]=1;
                }
                if(i+1<nb_c*nb_l && t_max*d_threshold<s_buf[i+1])
                {
                    matrixFilter[i+1]=1;
                }
            }
        }
    }
}

int DataFILEClass::getTime()
{
    return timedata.front();
}
