/*
 * DataPlayfield.cpp
 *
 *  Created on: Mar 22, 2010
 *      Author: roger
 */

#include "DataPlayfield.h"

/* Constructor:
 * anb_l/anb_c number of lines & columns of finger matrix
 * aFile file saving spikes
 */
DataPlayfield::DataPlayfield(int anb_l,int anb_c,string aFile) {
    /* Variables definition */
    nb_l = anb_l;
    nb_c = anb_c;
    outputFile = aFile;
    currentTime = 0;        //now useless?
    periodAcq = 0.03;       //now useless?
    rr = gaussian_radius;                       // gaussian kernel first radius
    rr2 = gaussian_radius2;                     // gaussian kernel second radius
    timeWindow=5.00;        //now useless?
    multFactor = synapse_sensibility;		// program arguments, to be modified
    multNoise = 0.3;        //now useless?
    nbOfAfferents = nb_l*nb_c;      // ie number of mecanoreceptors
    simulationTime= (float)(acq_period)/1000;                       // time between two acquisitions (in sec)

    /* preparing networks */
    listOfKernels = computeListOfGaussianFullKernel(nb_l,nb_c,rr,rr2);
    test = new TestAfferents(outputFile, nbOfAfferents, simulationTime);		// definition of a neurons
}

/* Destructor */
DataPlayfield::~DataPlayfield() {
    delete test;
    for (int i = 0; i < NBL; i++  ){
        delete[] lastData[i];
    }
//    delete lastData;
}

/* first call of class simulation */
void DataPlayfield::firstRun(double ** dataTab)
{
    prepareNextRun(dataTab);        // saves matrix to memory
}

/* prepares class for next simulation run */
void DataPlayfield::prepareNextRun(double** dataTab)
{
    lastData = dataTab;                         // saves matrix to memory
    currentTime = currentTime + periodAcq;	// updates clock (still necessary ??)
}

/* main simulation method */
void DataPlayfield::play(double** dataTab)
{
    TemporalSkinData skinData(nb_l,nb_c);                       // creates class to contain data
    skinData.addMatrix(lastData);				// adds the old matrix to data
    skinData.addMatrix(dataTab);				// adds new matrix to data
    skinData.multiply(multFactor);				// Multiply by factor (program argument)

//    TemporalSkinData * skinData2 = skinData.getInterpolatedData(46);					// Adds intermediary stages of data
    TemporalSkinData * skinData2 = skinData.getInterpolatedData(TimeSaveStruct->time_new-TimeSaveStruct->time_old-1);
//    std::cout << TimeSaveStruct->time_new-TimeSaveStruct->time_old-1 << std::endl;
    vector<vector<double> > * tempDataList= computeTemporalDataList(*skinData2,listOfKernels);		// Application of Gausian Kernel filter

    test->setTemporalDataList(tempDataList);                                                            // adds data to network class
    test->resetTime();											// resets algorithm current step
    test->resetSpikeTimer();                                                                            // resets increments that are used to estimate spike times
    test->run();											// running of synapse simulation => neuron activity (spikes)

    prepareNextRun(dataTab);                                    // saves pertinent data for next run

    // freeing memory
    for(int i=0;i<tempDataList->size();i++)
    {
//        tempDataList[0].clear();
        tempDataList[i].clear();
    }
    tempDataList->clear();
    delete tempDataList;
    delete skinData2;
}
