#include "cerebellumcomm.h"

CerebellumComm::CerebellumComm()
{
    lastSimCmdTime = -acq_period;
    currentTime = 0;
    CerebellumSocket = NULL;
}

CerebellumComm::~CerebellumComm()
{
    if(CerebellumSocket == NULL)
        delete CerebellumSocket;
}

void CerebellumComm::run()
{
    initSocketTCP();

    cerebellumCommType CommVarRec;

    while(true)
    {
        this->setTerminationEnabled(false);
        CommVarRec = this->RecFromCerebellum();
        if(CommVarRec.time > currentTime)
        {
            currentTime = CommVarRec.time;
            if(currentTime*1000 >= lastSimCmdTime*1000 + acq_period)
            {
                this->Send2fingerSim(CommVarRec);
                lastSimCmdTime = currentTime;
            }

            emit(updateArmStateInfo(CommVarRec));
//            this->Send2cerebelum();     //!!!!!!!!!!!!!!!!!!!!!!! to be activated by signal slot
        }
        this->setTerminationEnabled(true);
    }
}

void CerebellumComm::initSocketTCP()
{
    if(CerebellumSocket == NULL)
    {
        string server_address = string(CerebellumAddress);
        unsigned short tcp_port = atoi(SendCerebellumPort);
        cout << "Trying to connect to SERVER: " << server_address << " port " << tcp_port << endl;
        CerebellumSocket = new Cd_Socket(CLIENT,/*UNIX*/INET_FAMILY,server_address,tcp_port);
    }
}


void CerebellumComm::Send2cerebelum()
{
    cerebellumCommType tmp = this->getLastCommand();
    if(tmp.time<0)
    {
        tmp.statusflag = 0;
    }
    else
    {
        tmp.statusflag = 1;
    }
    tmp.time = currentTime;
    cout << "Sending FLAG : " << tmp.statusflag << endl;
    CerebellumSocket->sendBuffer(&tmp,sizeof(tmp));
}

cerebellumCommType CerebellumComm::RecFromCerebellum()
{
    int msg;        // serves for ?? - to check
    cerebellumCommType tmp;

    msg = CerebellumSocket->receiveBuffer(&tmp,sizeof(tmp));
    if(msg>0)
    {
        cout << "TIME (in s):" << tmp.time << endl;
    }

    return tmp;
}

void CerebellumComm::Send2fingerSim(cerebellumCommType CMD)
{
    if(useFILE==3)          // ie: if using fingerSim
    {
        fingerSimCmdType tmp;
        tmp.time = CMD.time;
        tmp.posX = CMD.posX;
        tmp.posY = CMD.posY;

        std::string servAddress = SimulatorAddress;
        unsigned short echoServPort = Socket::resolveService(SendFingerSIMPort,"udp");

        try
        {
            UDPSocket sock;
            sock.sendTo(&tmp, sizeof(tmp), servAddress, echoServPort);
        }
        catch (SocketException &e)
        {
            cout << e.what() << endl;
            return;
        }
    }
    return;
}

cerebellumCommType CerebellumComm::getLastCommand()
{
    int Mintime = currentTime*1000 - CerebellumCommDelay;           // Careful ms && s (to be checked !!)

    cerebellumCommType tmp;

    if(!CommVect.isEmpty())
    {
        if(CommVect.front().time<= Mintime)
        {
            tmp = CommVect.front();
            CommVect.removeFirst();
        }
    }
    return tmp;
}

void CerebellumComm::clean()
{
    CommVect.clear();
    lastSimCmdTime = -acq_period;
    currentTime = 0;
}

void CerebellumComm::add2fifo(double posX, double posY, double vX, double vY, double aX, double aY,double robotFlag)
{
    cerebellumCommType tmp;
    tmp.time = currentTime;
    tmp.aX = aX;
    tmp.aY = aY;
    tmp.posX = posX;
    tmp.posY = posY;
    tmp.vX = vX;
    tmp.vY = vY;
    tmp.statusflag = robotFlag;

    CommVect.push_back(tmp);
}

void CerebellumComm::OperationCompleteFeedbackReceived()
{
    this->Send2cerebelum();
}
