//( directafferents.hpp

#ifndef DIRECTAFFERENTS_HPP
#define DIRECTAFFERENTS_HPP

#include "neuralnetwork.hpp"
#include <assert.h>
#include <iostream>
#include <math.h>
#include <list>
#include <vector>

//( DirectAfferents

/**
 * Network simulating the first layer of neuron with the intensity inputs from the artificial skin
 *
 * @author Xavier
 **/
class DirectAfferents :
  public NeuralNetwork
{
public:
  /**
   *
   * Constructor
   *
   * @param   numberOfPyramidalCell  number of pyramidal cell in the network
   *
   **/
  DirectAfferents(const int numberOfAfferents,bool spikePreLearning);

  /**
   * Default destructor
   *
   **/
  ~DirectAfferents();

public:
  /**
   *
   * Set a stimulus for the next step of the network
   *
   * @param   inputs  inputs of the artificial skin
   *
   * @return  void
   *
   **/
  void setStimulus(vector<double> & inputs);

  /**
   *
   * Apply the same input everywhere
   *
   * @param   input the unique input
   *
   * @return  void
   *
   **/
  void setStimulus(double input);


  /**
   *
   *
   * @return  the main vector of neurons
   *
   **/
  NeuronsVector & getMainNeuronsVector();

  /**
   *
   *
   * @return  the vector of afferent neurons
   *
   **/
  NeuronsVector & getAfferentNeuronsVector();


    /**
    *
    *
    * @return the raster info of the neurons
    *
    *
    **/
  vector< list<double> > getSpikeTimeList();



  void printAllWeights(ofstream & weightsStream);


protected:
  /**
   *
   * Vector of pyramidal cells
   *
   **/
  NeuronsVector lifVector;

}
;

//) DirectAfferents


#endif

//) directafferents.hpp
