#ifndef GLOBALVAR_H
#define GLOBALVAR_H

#include <QSemaphore>
#include <iostream>
#include <QVector>
#include "edlut.h"
#include "timesave.h"

//#define address "localhost" // "192.168.1.12.0.0.1"          // To be added to config file
//#define rsp_port "64501"
//#define cmd_port "64500"

extern QSemaphore *writingSemaphore;
extern QSemaphore *readingSemaphore;
extern double** sharedData;
extern int sharedTime;
extern bool readingThread_isrunning;
extern bool writingThread_isrunning;
extern QSemaphore *StoreDisplaySemaphore;
extern QSemaphore *edlutSemaphore;

extern bool saveDataOK;
extern QSemaphore *saveDataOKSemaphore;
extern bool send2MatlabOK;
extern QSemaphore * send2MatlabOKSemaphore;
//extern bool send2RobotOK;
//extern QSemaphore * send2RobotOKSemaphore;
extern QSemaphore * useControlOKSemaphore;
extern bool useControlOK;

extern QVector<int> *DisplaySpikeData;
extern QVector<int> *DisplayTimeData;


extern EDLUT * eSim;
extern TimeSave * TimeSaveStruct;

double ** init_sharedData();
void init_semaphores();
void init_sharedTime();
void init_sharedDataVars();


// struct of commands to be sent to robot
struct RobotCmdType
{
    RobotCmdType()
    {
        clean();
    };
    void clean()
    {
        timestamp=0;
        statusflag=0;
        vx=0;
        vy=0;
        vz=0;
        wx=0;
        wy=0;
        wz=0;
    }
    double timestamp;       // to check for loss of information
    double statusflag;      // 0=wait | 1=initial positioning | 2=move | -1=reset | -2=endline
    double posX;
    double posY;
    double posZ;
    double time;
    double vx;              // in m/s
    double vy;              // in m/s
    double vz;              // in m/s
    double wx;              // rotation speed around x axis (pitch)
    double wy;              // rotation speed around y axis (roll)
    double wz;              // rotation speed around z axis (yaw)
};

//// struct of responses to be send by robot to supervising program
//struct rspType
//{
//    rspType()
//    {
//        clean();
//    };
//    void clean()
//    {
//        timestamp=0;
//        Rstatus=0;
//        xpos=0;
//        ypos=0;
//        zpos=0;
//        xspeed=0;
//        yspeed=0;
//        zspeed=0;
//    }
//    double timestamp;       // to check for loss of information
//    double Rstatus;         // robot status (-1=not listening,not ready,positioning | 1=moving,ready,listening | 0=waiting)
//    double xpos;            // in m
//    double ypos;            // in m
//    double zpos;            // in m
//    double xspeed;          // in m/s
//    double yspeed;          // in m/s
//    double zspeed;          // in m/s
//};

struct MatlabRspType
{
        MatlabRspType()
        {
            clean();
        };
        void clean()
        {
            timeFlag = 0;
            detectTOP = 0;
            insClass = 0;
            RobotFlag = 0;
            vx = 0;
            vy = 0;
            vz = 0;
        }
        int timeFlag;
        int detectTOP;  // if classification has been made
        int insClass;   // which class is currently most probable
        int RobotFlag;   // movement flag of robot (to see if stop/backwards movement necessary
        int vx;
        int vy;
        int vz;
};

extern QSemaphore * doRecordSemaphore;
extern bool doRecord;

//// semaphore for sharing robot response between two threads
//extern QSemaphore* sharedRSPSemaphore;
//extern rspType* sharedRSP;

#endif // GLOBALVAR_H
