//( lifneuron.cpp


#include <iostream>
#include <algorithm>
#include <math.h>
#include <assert.h>

#include "parameters.hpp"
#include "hebblearningparameters.hpp"
#include "lifneuron.hpp"
#include "math.hpp"

#define max(i,j) (i)<(j)?j:i
#define min(i,j) (i)<(j)?i:j

//( LifNeuron



LifNeuron::LifNeuron(double resetPotential, double restingPotential,
		     double firingThreshold, double membraneCapacitance,
		     double leakConductance, double refractoryPeriod,
		     bool hebbLearning,
		     bool presynapticLearning, bool postsynapticLearning)
{
  assert(!(hebbLearning && (presynapticLearning || postsynapticLearning)));
  intensityStatisticArray = 0;

  this->resetPotential = resetPotential;
  this->restingPotential = restingPotential;
  this->firingThreshold = firingThreshold;
  this->membraneCapacitance = membraneCapacitance;
  this->leakConductance = leakConductance;
  this->refractoryPeriod = refractoryPeriod;

  //currentValue = Math::random()*(firingThreshold-restingPotential)+restingPotential;
  currentValue = restingPotential;
  inputTestingIntensity = 0;

  this->hebbLearning = hebbLearning;
  this->presynapticLearning = presynapticLearning;
  this->postsynapticLearning = postsynapticLearning;
  if(presynapticLearning) presynapticFunction = new PresynapticFunction(this);
  if(postsynapticLearning) postsynapticFunction = new PostsynapticFunction(this);

  equilibrium = 0;
}


LifNeuron::~LifNeuron()
{
	//Modif boiteuse... ou pas
 if(presynapticLearning) delete presynapticFunction;
 if(postsynapticLearning) delete postsynapticFunction;

}


double LifNeuron::getCurrentPotential() const
{
  return currentValue;
}


double LifNeuron::getCurrentSynapticIntensity(RungeKutta rungeKutta) const
{
  double currentSynapticIntensity = 0;

  for (unsigned int index = 0;
       index < synapseList.size();
       index++)
    {
      currentSynapticIntensity += ( synapseList[index]->getGeneratedIntensity(this, rungeKutta)
				    * synapseList[index]->getWeight() );
    }
  return currentSynapticIntensity;
}


double LifNeuron::getCurrentInputIntensity(RungeKutta rungeKutta) const
{
  return (getCurrentSynapticIntensity(rungeKutta) + inputTestingIntensity);
}


void LifNeuron::computeOneStep()
{
    if (!isFiring())
    {
        if (canFire())
        {
            generateSpike();
        }
        else
        {
            computeNextStepRK2(timeStep);
            if (noiseFlag)
            {
                double interval = firingThreshold - restingPotential;
                double noiseInterval = interval * internNoiseIntensity;
                double variation = (noiseInterval * 2 * Math::random()) - noiseInterval;

                currentValue += variation;
            }
            fillIntensityStatisticArray();
        }
    }
    inputTestingIntensity = 0;

      //  Hebbian Learning rate-based
//    if(hebbLearning && ((static_cast<int>(getCurrentTime()/timeStep)%(static_cast<int>(rateTimeWindow/timeStep))) == 0) )
//    {
//          //        cout << "currentTime : " << getCurrentTime() << endl;
//        computeRateBasedLearning();
//    }

  //  Hebbian Learning spike-based
    if(postsynapticLearning)
        postsynapticFunction->nextStep();

    if(presynapticLearning)
    {
        presynapticFunction->nextStep();
        LifNeuron * efferentNeuron;
        float * w_temp;
        if(this->isFiring())
        {
            for(PresynapticConnectionList::iterator iterator=presynapticConnections.begin();
                iterator != presynapticConnections.end();
                iterator++)
            {
                efferentNeuron = (*iterator)->synapse->getEfferentNeuron();
//
//                if ((*iterator)->synapse->getReceptorType() == Synapse::GABAReceptor)
//                    cout <<"GABA in learning !!" <<endl;

//                if ((*iterator)->synapse->getReceptorType() == Synapse::AMPAReceptor)
//                    cout <<"AMPA in learning !!" <<endl;

//                if ((*iterator)->synapse->getReceptorType() == Synapse::NMDAReceptor)
//                    cout <<"NMDA in learning !!" <<endl;

                (*iterator)->weight -= efferentNeuron->postsynapticFunction->getCurrentValue()*((*iterator)->weight);//-w_in;///*+0.00100*/;

                if((*iterator)->weight<0)
                    (*iterator)->weight=0.0;
            }
        }
        else
        {
            for(int index = 0;
                index < (*efferentFiringPositions)[0];
                index ++)
            {
                w_temp = & presynapticConnections[(*efferentFiringPositions)[index+1]]->weight;
                *w_temp += this->presynapticFunction->getCurrentValue();

                if((*w_temp)>maxWeight)
                {
                    *w_temp=maxWeight;
                }
                //if(*w_temp<0)
                  //  *w_temp=0;
            }
        }
    }
// // balance of the weights
//        if( (static_cast<int>(getCurrentTime()/timeStep) % 1000) == 0)
//  	{
//  	  for(int index = 0;
//  	      index < presynapticConnections.size();
//  	      index ++)
//  	    {
//  	      double w = presynapticConnections[index]->weight;
//  	      if(w>0 && w<maxWeight)
//  		presynapticConnections[index]->weight -= (equilibrium)/((double)presynapticConnections.size());
//  	    }
//  	}


////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
//with learning window
// if(presynapticLearning)
// {
//   if(this->isFiring())
//   {
//     for(PresynapticConnectionList::iterator iterator=presynapticConnections.begin();   //neuron presynaptic
//         iterator != presynapticConnections.end();
//         iterator++)
//     {
//       (*iterator)->weight += learningWindow(getCurrentTime()-(*iterator)->synapse->getEfferentNeuron()->getLastSpikeTime());
//       if((*iterator)->weight<0)
//         (*iterator)->weight=0;
//       if((*iterator)->weight>maxWeight)
//         (*iterator)->weight=maxWeight;
//     }
//   }
//   else
//   {
//     float * w_temp;
//     for(int index = 0;                           //neuron postsynaptic
//         index < (*efferentFiringPositions)[0];
//         index ++)
//     {
//       w_temp = & presynapticConnections[(*efferentFiringPositions)[index+1]]->weight;
//       *w_temp += learningWindow(getCurrentTime()-this->getLastSpikeTime())-0.050;
//       if(*w_temp<0)
//         *w_temp=0;
//       if(*w_temp>maxWeight)
//         *w_temp=maxWeight;
//     }
//   }
// }

}


bool LifNeuron::canFire() const
{
  if ( !spikeTimeList.empty() )
    {
      if ( (getCurrentTime() - spikeTimeList.back()) <= refractoryPeriod )
      {
        return false;
      }
    }
  if (currentValue > firingThreshold)
    {
      return true;
    }

  return false;
}


void LifNeuron::generateSpike()
{
  currentValue = resetPotential;
  spikeTimeList.push_back(getCurrentTime());
  FiringNeuron::generateSpike();

  //cout << "Firing" <<endl;

}


double LifNeuron::computeDerivate(double currentValue, RungeKutta rungeKutta)
{
  return (- leakConductance * (currentValue - restingPotential)
	  - getCurrentInputIntensity(rungeKutta)) / membraneCapacitance;
}


bool LifNeuron::isFiring() const
{
  if ( !spikeTimeList.empty() )
    {
      return ((getCurrentTime() + (timeStep / 2)) < (spikeTimeList.back() + timeStep));
    }
  else
    {
      return false;
    }
}


void LifNeuron::addReceptor(Synapse * synapse)
{
  assert(synapse != 0);
  SynapseList::iterator iterator = find(synapseList.begin(), synapseList.end(), synapse);
  if (iterator == synapseList.end())
    {
      synapseList.push_back(synapse);
    }
}



void LifNeuron::setInputTestingIntensity(double intensity)
{
  inputTestingIntensity = intensity;
}


Synapse * LifNeuron::getReceptor(const Synapse::ReceptorType receptorType)
{
  Synapse * result = 0;
  SynapseList::reverse_iterator iterator = synapseList.rbegin();
  while ( (result == 0)
	  && (iterator != synapseList.rend()) )
    {
      if ( (*iterator)->getReceptorType() == receptorType )
	{
	  return (*iterator);
	}
      iterator++;
    }
  return result;
}


void LifNeuron::setIntensityStatisticArray(IntensityArray * intensityArray)
{
  this->intensityStatisticArray = intensityArray;
  if (intensityStatisticArray)
    {
      intensityStatisticArray->resize(synapseList.size());
    }
}


void LifNeuron::fillIntensityStatisticArray()
{
  if (intensityStatisticArray)
    {
      for (unsigned int index = 0;
	   index < synapseList.size();
	   index++)
	{
 	  (*intensityStatisticArray)[index] = ( synapseList[index]->getGeneratedIntensity(this)
						* synapseList[index]->getWeight() );
	}
    }
}


void LifNeuron::getFiringTimes(double timeWindow, SpikeTimeList & lastFiringTimes)
{
  SpikeTimeList::reverse_iterator iterator = spikeTimeList.rbegin();

  if(this->spikeTimeList.size()==0) return;

  for(; (*iterator)>(getCurrentTime()-timeWindow) && iterator!=spikeTimeList.rend() ; iterator++)
    {
      lastFiringTimes.push_back(*iterator);
    }
}


void LifNeuron::computeRateBasedLearning()
{
  PresynapticConnectionList::iterator iterator = presynapticConnections.begin();

  Synapse * synapse;
  LifNeuron * efferent;
  double synapseWeight;
  double modifiedSynapseWeight;
  double afferentRate = this->getNbOfRecentSpikes(rateTimeWindow);
  afferentRate /= rateTimeWindow;
  double efferentRate = 0;


  for(unsigned int index = 0; index < presynapticConnections.size(); index++)
    {
      synapse = (*iterator)->synapse;
      synapseWeight = (*iterator)->weight;
      efferent = synapse->getEfferentNeuron();
      efferentRate = efferent->getNbOfRecentSpikes(rateTimeWindow);
      efferentRate /= rateTimeWindow;

      modifiedSynapseWeight = computeRateBasedLearning(synapseWeight, afferentRate, efferentRate);

      (*iterator)->setConnectionWeight(modifiedSynapseWeight);

      iterator++;
    }
}


double LifNeuron::computeRateBasedLearning(double weight, double afferentRate, double efferentRate)
{
  // min-max threshold rule
  double maxWeight = 2;
  double maxRate = 100;
  //    double delta_weight = (maxWeight-weight)*oja_eta0 * efferentRate * afferentRate - weight_decrease ;
  double delta_weight = (maxWeight-weight)*oja_eta0 * efferentRate * afferentRate - weight_decrease * (efferentRate / maxRate) ;
  // Oja Rule
  //    double delta_weight = oja_eta0 * efferentRate * ( afferentRate - efferentRate*weight);
  //    cout << modifiedSynapseWeight << endl;
  return min(max(weight + delta_weight,0),maxWeight);
}



int LifNeuron::getNbOfRecentSpikes(double timeGap)
{
  SpikeTimeList::reverse_iterator iterator = spikeTimeList.rbegin();

  if(this->spikeTimeList.size()==0) return 0;
  int nbOfSpikes = 0;
  for(; (*iterator)>(getCurrentTime()-timeGap) && iterator!=spikeTimeList.rend() ; iterator++)
    {
      nbOfSpikes ++;
    }
  return nbOfSpikes;
}


//( PresynapticFunction
LifNeuron::PresynapticFunction::PresynapticFunction(LifNeuron * parent)
{
  this->parent = parent;
  currentValue = 0;
}

void LifNeuron::PresynapticFunction::computeOneStep()
{
  if(this->parent->isFiring()) currentValue = A_positive;//+=
  computeNextStepRK2(timeStep);
}

double LifNeuron::PresynapticFunction::computeDerivate(double currentValue, RungeKutta rungeKutta)
{
  return - currentValue / t_positive;
}

//) PresynapticFunction


//( PostsynapticFunction
LifNeuron::PostsynapticFunction::PostsynapticFunction(LifNeuron * parent)
{
  this->parent = parent;
  currentValue = 0;
}

void LifNeuron::PostsynapticFunction::computeOneStep()
{
  if(this->parent->isFiring()) currentValue = A_negative;//+=
  computeNextStepRK2(timeStep);
}

double LifNeuron::PostsynapticFunction::computeDerivate(double currentValue, RungeKutta rungeKutta)
{
  return - currentValue / t_negative;
}

//) PostsynapticFunction


double LifNeuron::getPresynapticFunctionValue()
{
  return this->presynapticFunction->getCurrentValue();
}


double LifNeuron::getPostsynapticFunctionValue()
{
  return this->postsynapticFunction->getCurrentValue();
}

double LifNeuron::getLastSpikeTime()
{
  if(this->spikeTimeList.empty())
    return -100.0;
  else
    return this->spikeTimeList.back();
}

double LifNeuron::learningWindow(double timeGap)
{
  const double A=0.50;//A=0.05;
  const double t2=0.0330;//0.050
  const double to=0.0010;
  //const double constDecrease = 0.100;

  double t = fabs(timeGap);

  if (t<t2)
  {
    return A*cos(t*M_PI/t2);
  }
  else
  {
    //return -constDecrease+(constDecrease-A)*exp((t2-t)/to);
    return -A*exp((t2-t)/to);
  }
}

//) LifNeuron


//) lifneuron.cpp
