//( math.cpp

#include <algorithm>
#include <math.h>

#include "math.hpp"


//( Math


long Math::idum = 0;


double Math::gaussianDistribution(double jmoins, double jplus, 
				  double standardDeviation, 
				  double x, double maximum) 
{
  x = x - maximum;
  return (jmoins + (jplus - jmoins) 
	  * exp( -(x * x) / (2 * standardDeviation * standardDeviation)));
}


double Math::poissonDistribution(const double frequency, const double timeStep)
{
  //return 0;
  //return exp(-(frequency * timeStep)) * frequency * timeStep;
  return frequency * timeStep;
}

double Math::getDistanceInCircularRange(double originValue, 
					double targetValue, 
					double rangeSize) 
{
  double result = fabs(originValue - targetValue);
  if (result > (rangeSize / 2))
    {
      result = rangeSize - result;
    }
  return  result;
}



double Math::random() 
{
  const int IA = 16807;
  const int IM = 2147483647;
  const double AM = (1.0/IM);
  const int IQ = 127773;
  const int IR = 2836;
  const int NTAB = 32;
  const int NDIV = (1+(IM-1)/NTAB);
  const double EPS = 1.2e-7;
  const double RNMX = (1.0-EPS);

  int j;
  long k;
  static long iy=0;
  static long iv[NTAB];
  float temp;
  
  if (idum <= 0 || !iy) {
    if (-(idum) < 1) idum=1;
    else idum = -(idum);
    for (j=NTAB+7;j>=0;j--) {
      k=(idum)/IQ;
      idum=IA*(idum-k*IQ)-IR*k;
      if (idum < 0) idum += IM;
      if (j < NTAB) iv[j] = idum;
    }
    iy=iv[0];
  }
  k=(idum)/IQ;
  idum=IA*(idum-k*IQ)-IR*k;
  if (idum < 0) idum += IM;
  j=iy/NDIV;
  iy=iv[j];
  iv[j] = idum;
  if ((temp=AM*iy) > RNMX) return RNMX;
  else return temp;
}


void Math::setSeed(long startValue) 
{
  idum = startValue;
}


float gammln(float xx)
{
  double x,y,tmp,ser;
  static double cof[6]={76.18009172947146,-86.50532032941677,
			24.01409824083091,-1.231739572450155,
			0.1208650973866179e-2,-0.5395239384953e-5};
  int j;
  
  y=x=xx;
  tmp=x+53.5;
  tmp -= (x+0.5)*log(tmp);
  ser=1.000000000190015;
  for (j=0;j<=5;j++) ser += cof[j]/++y;
  return -tmp+log(2.5066282746310005*ser/x);
}


float Math::poidev(float xm) 
{
  static float sq,alxm,g,oldm=(-1.0);
  float em,t,y;
  
  if (xm < 12.0) {
    if (xm != oldm) {
      oldm=xm;
      g=exp(-xm);
    }
    em = -1;
    t=1.0;
    do {
      ++em;
      t *= random();
    } while (t > g);
  } else {
    if (xm != oldm) {
      oldm=xm;
      sq=sqrt(2.0*xm);
      alxm=log(xm);
      g=xm*alxm-gammln(xm+1.0);
    }
    do {
      do {
	y=tan(M_PI*random());
	em=sq*y+xm;
      } while (em < 0.0);
      em=floor(em);
      t=0.9*(1.0+y*y)*exp(em*alxm-gammln(em+1.0)-g);
    } while (random() > t);
  }
  return em;
}

//) Math


//) math.cpp
