//( math.hpp

#ifndef MATH_HPP
#define MATH_HPP



//( Math

/**
 * Class for useful function in mathematique
 *
 * @author Thomas
 **/
class Math 
{
public:
  /**
   *
   * Gaussian distribution between jmoins and jplus
   * 
   * @param   jmoins             minimum value of the distribution
   * @param   jplus              maximum value of the distribution
   * @param   standardDeviation  standard deviation of the distribution
   * @param   x                  x
   * @param   maximum            where the curve is maximum (by default, it is centered on 0)
   *
   * @return  x distributed with a gaussian distribution
   *
   **/
  static double gaussianDistribution(double jmoins, double jplus, 
				     double standardDeviation, 
				     double x, double maximum = 0);

  /**
   *
   * Distribution de Poisson
   * 
   * @param   frequency  frequency of the event ( = probability) (Hertz)
   * @param   timeStep   time step to test the probability of the event
   *
   * @return  the poisson distribution
   *
   **/
  static double poissonDistribution(const double frequency, const double timeStep);

  /**
   *
   * Compute a distance between two value on a circular range
   * 
   * @param   originValue  the first value to compare
   * @param   targetValue  the second value to compare
   * @param   rangeSize    the size of the range
   *
   * @return  the distance computed
   *
   **/
  static double getDistanceInCircularRange(double originValue, 
					   double targetValue, 
					   double rangeSize);

  /**
   *
   *
   * @return  a pseudo-random number between 0 and 1
   *
   **/
  static double random();

  /**
   *
   * Set the first value of the random seed 
   * 
   * @param   startValue  start value of the random process
   *
   * @return  void
   *
   **/
  static void setSeed(long startValue);

  /**
   *
   * Poisson distribution
   * 
   * @param   xm  poisson parameter
   *
   * @return  random event
   *
   **/
  static float poidev(float xm);


private:
  /**
   *
   * Used to compute the random value extracted
   * 
   **/
  static long idum;

}
;

//) Math


#endif

//) math.hpp
