//( mylifneuron.cpp


#include "mylifneuron.hpp"
#include "parameters.hpp"

//( MyLifNeuron

MyLifNeuron::MyLifNeuron(bool hebbLearning, bool presynapticLearning, bool postsynapticLearning) :
  LifNeuron(myLifNeuronResetPotential, myLifNeuronRestingPotential,
	    myLifNeuronFiringThreshold, myLifNeuronMembraneCapacitance,
	    myLifNeuronLeakConductance, myLifNeuronRefractoryPeriod, hebbLearning, presynapticLearning, postsynapticLearning)
{
    initialFiringThreshold=myLifNeuronFiringThreshold;
    thresFunction=new ThresholdFunction(this);
}


MyLifNeuron::~MyLifNeuron()
{
    delete thresFunction;
}


LifNeuron::NeuronType MyLifNeuron::getNeuronType() const
{
  return LifNeuron::MyLifNeuron;
}


FiringNeuron * MyLifNeuron::createNeuron(bool hebbLearning, bool presynapticLearning, bool postsynapticLearning) const
{
  return new MyLifNeuron(hebbLearning, presynapticLearning, postsynapticLearning);
}


void MyLifNeuron::setStimulus(const double intensity)
{
    LifNeuron::setInputTestingIntensity(intensity);
}


void MyLifNeuron::computeOneStep()
{
    firingThreshold=thresFunction->getCurrentValue();
    LifNeuron::computeOneStep();
    thresFunction->nextStep();
}

//( ThresholdFunction
MyLifNeuron::ThresholdFunction::ThresholdFunction(MyLifNeuron * parent)
{
  this->parent = parent;
  currentValue = myLifNeuronRestingFiringThreshold;
}

void MyLifNeuron::ThresholdFunction::computeOneStep()
{
  if(this->parent->isFiring()) currentValue += A_threshold;//+=
  computeNextStepRK2(timeStep);
}

double MyLifNeuron::ThresholdFunction::computeDerivate(double currentValue, RungeKutta rungeKutta)
{
  return (myLifNeuronRestingFiringThreshold - currentValue) / tau_threshold;
}

//) ThresholdFunction

//) MyLifNeuron


//) mylifneuron.cpp
