//( mylifneuron.hpp

#ifndef MYLIFNEURON_HPP
#define MYLIFNEURON_HPP


#include "lifneuron.hpp"
#include "parameters.hpp"


//( MyLifNeuron

/**
 * custom lif neuron
 *
 * @author Xavier
 **/
class MyLifNeuron :
  public LifNeuron
{
public:
  /**
   * Default constructor
   *
   * @param hebbLearning   true if this neuron will perform hebbian learning
   *
   **/
  MyLifNeuron(bool hebbLearning = false, bool presynapticLearning = false, bool postsynapticLearning = false);

  /**
   * Default destructor
   *
   **/
  ~MyLifNeuron();

public:
  /**
   *
   *
   * @return  the type of this neuron
   *
   **/
  NeuronType getNeuronType() const;

public:
  /**
   *
   *
   * @return  a new neuron of the same nature of this one
   *
   **/
  FiringNeuron * createNeuron(bool hebbLearning = false, bool presynapticLearning = false, bool postsynapticLearning = false) const;


  void setStimulus(const double intensity);



private:

    /**
     *
     * Compute one more step for this object
     *
     *
     * @return  void
     *
     **/
    void computeOneStep();



    double initialFiringThreshold;


    private:
  /**
   *
   * class used to compute the postsynaptic function value for spike-dependant learning
   *
   **/
  class ThresholdFunction:
    public TimeDependantObject
  {
  public:
    /**
     * Constructor
     *
     * @param parent   the parent neuron of this function
     **/
    ThresholdFunction(MyLifNeuron * parent);

  protected:
    /**
     *
     * Compute one more step for this object
     *
     *
     * @return  void
     *
     **/
    void computeOneStep();

    /**
     *
     * Compute the derivate of this function
     *
     *
     * @param   currentValue  the last value of the function
     * @param   rungeKutta    indicate the step of the algorithm of Runge Kutta
     *
     * @return  the result of the computation
     *
     **/
    double computeDerivate(double currentValue, RungeKutta rungeKutta);

  private:
    /**
     *
     * Synapse using this instance
     *
     **/
    MyLifNeuron * parent;
  };

ThresholdFunction * thresFunction;

}
;

//) MyLifNeuron


#endif

//) mylifneuron.hpp
