#ifndef RECEIVEFROBOT_H
#define RECEIVEFROBOT_H

//#include "globalVar.h"
#include "PracticalSocket.h"
#include "parameters.hpp"
#include <QThread>
#include <QSemaphore>
#include <QObject>

// struct of responses to be send by robot to supervising program
struct rspType
{
    rspType()
    {
        clean();
    };
    void clean()
    {
        timestamp=0;
        Rstatus=0;
        xspeed=0;
        yspeed=0;
        zspeed=0;
        wx = 0;
        wy = 0;
        wz = 0;

        for(int i=0;i<9;i++)
            Rmat[i] = 0;

        xpos=0;
        ypos=0;
        zpos=0;
    }
    double timestamp;       // to check for loss of information
    double Rstatus;         // robot status (-1=not listening,not ready,positioning | 1=moving,ready,listening | 0=waiting)

    double xspeed;          // in m/s
    double yspeed;          // in m/s
    double zspeed;          // in m/s

    double wx;
    double wy;
    double wz;

    double Rmat[9];         // rotation matrix read column by column (for position)
    double xpos;            // in m
    double ypos;            // in m
    double zpos;            // in m
};
// semaphore for sharing robot response between two threads
extern QSemaphore* sharedRSPSemaphore;
extern rspType* sharedRSP;

extern bool send2RobotOK;
extern QSemaphore * send2RobotOKSemaphore;

class receiveFrobot : public QThread
{
    Q_OBJECT
public:
    rspType RSP;
    int TrialCounter;

    receiveFrobot();
    int listen2socket();
    virtual void run();

signals:
    void updateRobotFlag(double);     // signal for updating command to be sent to robot
    void nextLine();
    void posSIG(double);
    void PauseRec(bool);            // signals that the robot is returning (ie says to stop recording data)

};

#endif // RECEIVEFROBOT_H
