//( synapse.cpp

#include <stdlib.h>
#include <assert.h>

#include "synapse.hpp"
#include "math.hpp"
#include "lifneuron.hpp"
#include "parameters.hpp"


//( Synapse


Synapse::Synapse(ReceptorType receptor) :
  receptorType(receptor)
{
  weight = 1;
  recurrent = true;
  sumOfFiringNeurons = 0;
}


Synapse::~Synapse()
{
}


void Synapse::computeOneStep()
{
  computeNextStepRK2(timeStep);
  sumOfFiringNeurons = 0;
}


double Synapse::getGatingVariable() const
{
  return getCurrentValue();
}


Synapse::ReceptorType Synapse::getReceptorType() const
{
  return receptorType;
}


double Synapse::getGeneratedIntensity(const LifNeuron * postsynapticNeuron, RungeKutta rungeKutta) const
{
  return (getReceptorLeakConductance(postsynapticNeuron)
	  * (postsynapticNeuron->getCurrentValue(rungeKutta) - getReceptorReferencePotential())
	  * getCurrentValue(rungeKutta));
}


double Synapse::getWeight() const
{
  return weight;
}

void Synapse::setWeight(double weight)
{
  this->weight = weight;
}



bool Synapse::isRecurrent() const
{
  return recurrent;
}


void Synapse::setRecurrent(bool recurrent)
{
  this->recurrent = recurrent;
}

Synapse * Synapse::createSynapse() const
{
  Synapse * synapse = createNewSynapse();
  synapse->setWeight(weight);

  return synapse;
}

double Synapse::getSumFiringNeurons() const
{
  return sumOfFiringNeurons;
}

void Synapse::addConnection(FiringNeuron * presynapticNeuron, LifNeuron * postsynapticNeuron, double weight)
{
    if (presynapticNeuron != 0)
    {
        presynapticNeuron->addPresynapticConnection(this, weight);
    }
    if (postsynapticNeuron != 0)
    {
        postsynapticNeuron->addReceptor(this);
        this->efferent = postsynapticNeuron;
    }
}

void Synapse::sendSpike(double weight)
{
  sumOfFiringNeurons += weight;
}

LifNeuron *  Synapse::getEfferentNeuron()
{
  return efferent;
}


//) Synapse


//) synapse.cpp
