//( synapseampa.cpp


#include "synapseampa.hpp"
#include "parameters.hpp"
#include "lifneuron.hpp"

//( SynapseAMPA

SynapseAMPA::SynapseAMPA() :
  Synapse(AMPAReceptor)
{
}


SynapseAMPA::~SynapseAMPA()
{
}


double SynapseAMPA::computeDerivate(double currentValue, RungeKutta rungeKutta)
{
  return - (currentValue / ampaDecayTime) + (getSumFiringNeurons() / timeStep);
}


double SynapseAMPA::getReceptorReferencePotential() const
{
  return ampaReferencePotential;
}


double SynapseAMPA::getReceptorLeakConductance(const LifNeuron * postsynapticNeuron) const
{
  if (postsynapticNeuron->getNeuronType() == FiringNeuron::PyramidalCell)
    {
      if (recurrent)
	{
	  return ampaRecurrentPyramidalCellLeakConductance;
	}
      else
	{
	  return ampaExternPyramidalCellLeakConductance;
	}
    }
  else
    {
      if (recurrent)
	{
	  return ampaRecurrentInterneuronsLeakConductance;
	}
      else
	{
	  return ampaExternInterneuronsLeakConductance;
	}
    }
}

Synapse * SynapseAMPA::createNewSynapse() const
{
  return new SynapseAMPA();
}

//) SynapseAMPA


//) synapseampa.cpp
