//( synapsegaba.cpp


#include "synapsegaba.hpp"
#include "parameters.hpp"
#include "lifneuron.hpp"

//( SynapseGABA

SynapseGABA::SynapseGABA() :
  Synapse(GABAReceptor)
{
}


SynapseGABA::~SynapseGABA()
{
}


double SynapseGABA::computeDerivate(double currentValue, RungeKutta rungeKutta)
{
  return -(currentValue / gabaDecayTime) + getSumFiringNeurons() / timeStep;
}

double SynapseGABA::getReceptorReferencePotential() const
{
  return gabaReferencePotential;
}


double SynapseGABA::getReceptorLeakConductance(const LifNeuron * postsynapticNeuron) const
{
  if (postsynapticNeuron->getNeuronType() == FiringNeuron::PyramidalCell)
    {
      return gabaPyramidalCellLeakConductance;
    }
  else
    {
      return gabaInterneuronsLeakConductance;
    }
}


Synapse * SynapseGABA::createNewSynapse() const
{
  return new SynapseGABA();
}

double SynapseGABA::getGeneratedIntensity(const LifNeuron * postsynapticNeuron, RungeKutta rungeKutta) const
{
  return (getReceptorLeakConductance(postsynapticNeuron)
	  * max(0.0,(postsynapticNeuron->getCurrentValue(rungeKutta) - gabaReferencePotential))
	  * getCurrentValue(rungeKutta));
}


//) SynapseGABA


//) synapsegaba.cpp
