//( synapsenmda.cpp


#include "assert.h"
#include <math.h>

#include "synapsenmda.hpp"
#include "parameters.hpp"
#include "lifneuron.hpp"


//( SynapseNMDA

SynapseNMDA::SynapseNMDA() :
  Synapse(NMDAReceptor)//  ,
//    riseFunction(this)
{
}


SynapseNMDA::~SynapseNMDA()
{
}


double SynapseNMDA::computeDerivate(double currentValue, RungeKutta rungeKutta) 
{
  return ( - (currentValue / nmdaDecayTime) 
 	  + 0.025 * getSumFiringNeurons() / timeStep );//pkoi 0.025 ???? cf rapport
 //   return (- (currentValue / nmdaDecayTime) 
//   	  + nmdaAlpha * riseFunction.getCurrentValue(rungeKutta) * (1 - currentValue));
}


void SynapseNMDA::computeOneStep()
{
//    riseFunction.nextStep();
  Synapse::computeOneStep();
}


double SynapseNMDA::getReceptorReferencePotential() const
{
  return nmdaReferencePotential;
}


double SynapseNMDA::getReceptorLeakConductance(const LifNeuron * postsynapticNeuron) const
{
  if (postsynapticNeuron->getNeuronType() == FiringNeuron::PyramidalCell)
    {
      return nmdaPyramidalCellLeakConductance;
    }
  else
    {
      return nmdaInterneuronsLeakConductance;
    }
}


double SynapseNMDA::getGeneratedIntensity(const LifNeuron * postsynapticNeuron) const
{
  double temp;

  temp = (1 / (1 + 0.001 * exp(-0.062 * postsynapticNeuron->getCurrentPotential()) / 3.57) 
	  * getReceptorLeakConductance(postsynapticNeuron) 
	  * (postsynapticNeuron->getCurrentPotential() - getReceptorReferencePotential()) 
	  * getGatingVariable());

//   if (temp>0)
//     {
//       cout << "" << endl;
//       cout << postsynapticNeuron->getCurrentPotential() << endl;
//       cout << getReceptorLeakConductance(postsynapticNeuron) << endl;
//       cout << getReceptorReferencePotential() << endl;
//       cout << getGatingVariable() << endl;
//     }
  return temp;
}

//  double SynapseNMDA::getGeneratedIntensity(const LifNeuron * postsynapticNeuron) const
//  {
//    return (1 / (1 + 0.001 * exp(-0.062 * postsynapticNeuron->getCurrentPotential()) / 3.57) 
//  	  * getReceptorLeakConductance(postsynapticNeuron) 
//  	  * (postsynapticNeuron->getCurrentPotential() - getReceptorReferencePotential()) 
//  	  * getGatingVariable());
//  }

Synapse * SynapseNMDA::createNewSynapse() const
{
  return new SynapseNMDA();
}


//  //( SynapseNMDA::RiseFunction


//  SynapseNMDA::RiseFunction::RiseFunction(Synapse * synapse) 
//  {
//    this->parent = synapse;
//  }


//  void SynapseNMDA::RiseFunction::computeOneStep()
//  {
//    computeNextStepRK2(timeStep);
//  }


//  double SynapseNMDA::RiseFunction::computeDerivate(double currentValue, RungeKutta rungeKutta) 
//  {
//    return - (currentValue / nmdaRiseTime) + parent->getSumFiringNeurons() / timeStep;  

//  }


//) SynapseNMDA::RiseFunction


//) SynapseNMDA


//) synapsenmda.cpp
