//( temporalskindata.cpp



#include "temporalskindata.hpp"


//( TemporalSkinData

TemporalSkinData::TemporalSkinData(int s1,int s2,int nMatrix)
{
    size1=s1;
    size2=s2;
    nbOfMatrix=nMatrix;
}

TemporalSkinData::TemporalSkinData(int s1,int s2)
{
    size1=s1;
    size2=s2;
    nbOfMatrix = 0;
}

TemporalSkinData::~TemporalSkinData()
{
    for(int i=0;i<data.size();i++)
    {
        for(int j=0;j<data[i].size();j++)
        {
            data[i][j].clear();
        }
        data[i].clear();
    }
    data.clear();
}

void TemporalSkinData::addMatrix(double** dataTab)
{
    vector<vector<double> > buf;
    data.push_back(buf);

    for(int i=0;i<size1;i++)
    {
        vector<double> buf1;
        data[nbOfMatrix].push_back(buf1);

        for(int j=0;j<size2;j++)
        {
            data[nbOfMatrix][i].push_back(dataTab[i][j]);
        }
    }
    nbOfMatrix++;
}

TemporalSkinData * TemporalSkinData::getInterpolatedData(int nbOfIntermediateMatricesBetweenTwo)
{
    int newNbOfMat=(nbOfMatrix-1)*(nbOfIntermediateMatricesBetweenTwo+1)+1;
    TemporalSkinData * bufSD=new TemporalSkinData(size1,size2,newNbOfMat);


    for(int t=0;t<nbOfMatrix-1;t++)
    {
        for(int t2=0;t2<nbOfIntermediateMatricesBetweenTwo+1;t2++)
        {
            int matIndex=t*(nbOfIntermediateMatricesBetweenTwo+1)+t2;
            vector<vector<double> > buf4;
            bufSD->getDataVector().push_back(buf4);

            double interRatio=(double)t2/(nbOfIntermediateMatricesBetweenTwo+1);
            for(int i1=0;i1<size1;i1++)
            {
                vector<double> buf3;
                bufSD->getDataVector()[matIndex].push_back(buf3);
                for(int i2=0;i2<size2;i2++)
                {
                    (bufSD->getDataVector()[matIndex][i1]).push_back(data[t][i1][i2]*(1-interRatio)+data[t+1][i1][i2]*interRatio);
                }
            }
        }
    }


    vector<vector<double> > buf4;
    bufSD->getDataVector().push_back(buf4);

    for(int i1=0;i1<size1;i1++)
    {
        vector<double> buf3;
        bufSD->getDataVector()[newNbOfMat-1].push_back(buf3);
        for(int i2=0;i2<size2;i2++)
        {
            bufSD->getDataVector()[newNbOfMat-1][i1].push_back(data[nbOfMatrix-1][i1][i2]);
        }
    }


    return bufSD;
}

void TemporalSkinData::print()
{
    for(int m=0;m<nbOfMatrix;m++)
    {
        for(int i1=0;i1<size1;i1++)
        {
            for(int i2=0;i2<size2;i2++)
            {
                cout << data[m][i1][i2] << "\t";
            }
            cout <<endl;
        }
        cout <<endl;
    }

}

void TemporalSkinData::print(const string outputfile)
{
    ofstream outputFile(outputfile.c_str());

    outputFile << nbOfMatrix << "\t" << size1 << "\t" << size2 << "\n" <<endl;

    for (int i=0;i< nbOfMatrix;i++)
    {
        for (int j=0;j< size1;j++)
        {
            for (int k=0;k< size2;k++)
            {
                outputFile << data[i][j][k] << "\t";
            }
            outputFile << endl;
        }
        outputFile << endl;
    }

    outputFile.close();
}

void TemporalSkinData::printOutSum(const string outputfile)
{
    ofstream outputFile(outputfile.c_str());
    double buf;
    for (int i=0;i< nbOfMatrix;i++)
    {
        buf=0;
        for (int j=0;j< size1;j++)
        {
            for (int k=0;k< size2;k++)
            {
                buf+=data[i][j][k];
            }
        }
        outputFile << buf << endl;
    }

    outputFile.close();
}

void TemporalSkinData::printOutMax(const string outputfile)
{
    ofstream outputFile(outputfile.c_str());
    double buf;
    for (int i=0;i< nbOfMatrix;i++)
    {
        buf=0;
        for (int j=0;j< size1;j++)
        {
            for (int k=0;k< size2;k++)
            {
                buf=max(buf,data[i][j][k]);
            }
        }
        outputFile << buf << endl;
    }

    outputFile.close();
}

void TemporalSkinData::normalize()
{
    double sum,maxInput;
    for (int i=0;i< nbOfMatrix;i++)
    {
        sum=0;
        maxInput=0;
        for (int j=0;j< size1;j++)
        {
            for (int k=0;k< size2;k++)
            {
                maxInput=max(maxInput,data[i][j][k]);
                sum+=data[i][j][k];
            }
        }
        for (int j=0;j< size1;j++)
        {
            for (int k=0;k< size2;k++)
            {
                data[i][j][k]*=maxInput/sum;
            }
        }
    }
}

void TemporalSkinData::multiply(double factor)
{
    for (int i=0;i< nbOfMatrix;i++)
    {
        for (int j=0;j< size1;j++)
        {
            for (int k=0;k< size2;k++)
            {
                data[i][j][k]*=factor;
            }
        }
    }
}

void TemporalSkinData::addMultiplyingNoise(double noiseFactor)
{
    for (int i=0;i< nbOfMatrix;i++)
    {
        for (int j=0;j< size1;j++)
        {
            for (int k=0;k< size2;k++)
            {
                data[i][j][k]*= 1+ noiseFactor*(0.5-(double)rand()/RAND_MAX);
            }
        }
    }
}

vector<double> TemporalSkinData::applyFullKernel(vector<vector<double> > kernel)
{
    int kernelSize1=kernel.size();
    int kernelSize2=kernel[0].size();

    if ((size1!=kernelSize1)||(size2!=kernelSize2))
    {
        cout << "Attention la taille du noyau utilisé ne semble pas correspondre à la taille des données" << endl;
        cout << "size1" << size1 <<endl;
        cout << "kernelSize1" << kernelSize1 <<endl;
        cout << "size2" << size2 <<endl;
        cout << "kernelSize2" << kernelSize2 <<endl;
    }

    vector<double> result;

    double buf,sumKernel;
    sumKernel=0;
    for (int j=0;j< size1;j++)
    {
        for (int k=0;k< size2;k++)
        {
            sumKernel+=kernel[j][k];
        }
    }

    for (int i=0;i< nbOfMatrix;i++)				// for each data matrix
    {
        buf=0;							// application of filter
        for (int j=0;j< size1;j++)
        {
            for (int k=0;k< size2;k++)
            {
                buf+=data[i][j][k]*kernel[j][k];
            }
        }
        result.push_back(buf/sumKernel);                        // normalise results
    }

    return result;
}


// same as above but for one matrix (defined by its index)
double TemporalSkinData::applyFullKernel(vector<vector<double> > kernel,int matrixIndex)
{
    int kernelSize1=kernel.size();
    int kernelSize2=kernel[0].size();

    if ((size1!=kernelSize1)||(size2!=kernelSize2))
    {
        cout << "Attention la taille du noyau utilisé ne semble pas correspondre à la taille des données" << endl;
        cout << "size1 " << size1 <<endl;
        cout << "kernelSize1 " << kernelSize1 <<endl;
        cout << "size2 " << size2 <<endl;
        cout << "kernelSize2 " << kernelSize2 <<endl;
    }

    if ((matrixIndex>=nbOfMatrix)||(matrixIndex<0))
    {
        cout << "MatrixIndex invalide" << endl;
    }

    double buf,sumKernel;
    sumKernel=0;
    for (int j=0;j< size1;j++)
    {
        for (int k=0;k< size2;k++)
        {
            sumKernel+=kernel[j][k];
        }
    }

    buf=0;
    for (int j=0;j< size1;j++)
    {
        for (int k=0;k< size2;k++)
        {
            buf+=data[matrixIndex][j][k]*kernel[j][k];
        }
    }


    return (buf/sumKernel);
}

//) TemporalSkinData


//) temporalskindata.cpp
