//( timedependantobject.cpp


#include <iostream>
#include <assert.h>
#include <sys/time.h>


#include "timedependantobject.hpp"
#include "parameters.hpp"


//( TimeDependantObject


TimeDependantObject::TimeDependantObject()
{
  timeStep = ::timeStep;
  currentStep = 0;
  currentValue = 0;
  lastValue = 0;
  k1 = 0;
  k2= 0;
  preciseTime = boost::posix_time::ptime(boost::posix_time::microsec_clock::universal_time());
}


TimeDependantObject::~TimeDependantObject()
{
}


void TimeDependantObject::setTimeStep(double timeStep)
{
  TimeDependantObject::timeStep = timeStep;
}


double TimeDependantObject::getCurrentTime() const
{
  return currentStep * timeStep;
}


void TimeDependantObject::nextStep() 
{
  currentStep++;
  computeOneStep();
}


double TimeDependantObject::computeNextStepEuler(double timeStep) 
{
  lastValue = currentValue;
  currentValue = (lastValue + computeDerivate(lastValue) * timeStep);
  return currentValue;
}


double TimeDependantObject::computeNextStepRK2(double timeStep) 
{
  lastValue = currentValue;

  /*
   * Time interval
   */
  double h = timeStep;

  k1 = h * computeDerivate(currentValue, k1Step);

  k2 = h * computeDerivate(currentValue + k1 / 2, k2Step);

  currentValue = (currentValue + k2);

  return currentValue;
}


void TimeDependantObject::resetTime() 
{
  currentStep = 0;
}


double TimeDependantObject::computeDerivate(double currentValue, RungeKutta rungeKutta)
{
  assert(false);
  return 0;
}


void TimeDependantObject::startChrono() 
{
  chrono = getCurrentSystemTime();
}


double TimeDependantObject::getChrono() const
{
  return (getCurrentSystemTime() - chrono);
}


double TimeDependantObject::getCurrentSystemTime() const
{
  struct timeval timeOfTheDay;
  gettimeofday(&timeOfTheDay, 0);
  return timeOfTheDay.tv_sec + timeOfTheDay.tv_usec * 1e-6;
}


double TimeDependantObject::getTimeStep()
{
  return timeStep;
}


double TimeDependantObject::getCurrentValue(RungeKutta rungeKutta) const
{
  switch (rungeKutta)
    {
    k1Step:
      return lastValue;
    k2Step:
      return lastValue + k1 / 2;
    default:
      return currentValue;
    }
  assert(false);
}

//) TimeDependantObject


//) timedependantobject.cpp
