//( cd_socket.cpp

#include <iostream>
#include "cd_socket.hpp"
#include "utils/timer.hpp"
#define EXIT_FAILURE 1

/* exit example */
#include <stdio.h>
#include <stdlib.h>
#define SERVER_PATH "server"

//( Cd_Socket

Cd_Socket::Cd_Socket(unsigned short status, unsigned short family, unsigned short tcp_port)
{
  this->status = status;
  this->family = family;
  this->serv_tcp_port = tcp_port;
  this->serv_host_addr = "127.0.0.1";

  initializeSocket();
}

Cd_Socket::Cd_Socket(unsigned short status, unsigned short family, string server_address,unsigned short tcp_port)
{
  this->status = status;
  this->family = family;
  this->serv_host_addr = server_address;
  this->serv_tcp_port = tcp_port;

  initializeSocket();
}

Cd_Socket::~Cd_Socket()
{
  closesocket(socket_fd);
}


void Cd_Socket::initializeSocket()
{
  /***************/
  /*** CLIENT  ***/
  /***************/
  if(this->status==CLIENT)
    {
      // UNIX TYPE
      if(family==UNIX_FAMILY)
	{
// 	  cout << "Creating UNIX type client socket" << endl;

	  struct sockaddr_un serveraddr;

	  /********************************************************************/
	  /* The socket() function returns a socket descriptor representing   */
	  /* an endpoint.  The statement also identifies that the UNIX_CCSID  */
	  /* address family with the stream transport (SOCK_STREAM) will be   */
	  /* used for this socket.                                            */
	  /********************************************************************/
	  socket_fd = socket(AF_UNIX, SOCK_STREAM, 0); //_CCSID
	  if (socket_fd < 0)
	    {
	      perror("socket() failed");
	      return;
	    }

	  /********************************************************************/
	  /* If an argument was passed in, use this as the server, otherwise  */
	  /* use the #define that is located at the top of this program.      */
	  /********************************************************************/
	  memset(&serveraddr, 0, sizeof(serveraddr));
	  serveraddr.sun_family = AF_UNIX;
	  strcpy(serveraddr.sun_path, SERVER_PATH);

	  /********************************************************************/
	  /* Use the connect() function to establish a connection to the      */
	  /* server.                                                          */
	  /********************************************************************/
	  // introduce a timeout for the connection, let say 10 s
	  Timer * timer = new Timer();
	  double timeout = 10; // sec
	  int rc;
	  while(timer->getTimeS()<timeout)
	    {
	      rc = connect(socket_fd, (struct sockaddr *)&serveraddr, SUN_LEN(&serveraddr));
	      if(rc>=0)
		break;
	    }
	  delete timer;
	  if (rc < 0)
	    {
	      perror("connect() failed");
	      return;
	    }
	  else
	    ;
// 	    cout << "connection to server established" << endl;
	}

      // INET TYPE
      else
	{
// 	  cout << "Creating INET type client socket" << endl;

	  socket_fd = socket(AF_INET,SOCK_STREAM,0);
	  if(socket_fd == -1)
	    {
	      cerr << "Problem when creating the socket: creation ... closing the simulation (CD_SOCKET)" << endl;
	      exit(EXIT_FAILURE);
	    }
	  struct sockaddr_in serv_addr;
	  memset((char *)&serv_addr, 0, sizeof(serv_addr));
	  serv_addr.sin_family = AF_INET;
	  serv_addr.sin_addr.s_addr = inet_addr(this->serv_host_addr.c_str());
	  serv_addr.sin_port = htons(this->serv_tcp_port);
	  int socket_connect = connect(socket_fd,(struct sockaddr *)&serv_addr, sizeof(serv_addr));
	  int timer = 5;
	  int totalTime = 120;
	  while (socket_connect == -1 && totalTime > 0)
	    {
	      sleep (timer);
	      totalTime -= timer;
	      socket_connect = connect(socket_fd,(struct sockaddr *)&serv_addr, sizeof(serv_addr));
	      //exit(EXIT_FAILURE);
	    }
      if (socket_connect == -1 )
      {
          cerr << "Problem when connecting socket: connection ... closing the simulation ( (CD_SOCKET))" << endl;
          exit(EXIT_FAILURE);
      }
	  struct protoent *p;
	  int one=1;
	  p = getprotobyname("tcp");
	  setsockopt(socket_fd, p->p_proto, TCP_NODELAY, (const char*)&one, sizeof(one));
	}
    }


  /***************/
  /*** SERVER  ***/
  /***************/
  if(this->status==SERVER)
    {
      if(family==UNIX_FAMILY)
	{
// 	  cout << "Creating UNIX type socket ..." << endl;

	  /***********************************************************************/
	  /* Variable and structure definitions.                                 */
	  /***********************************************************************/
	  int    sd=-1, sd2=-1;
	  int    rc, length;
	  struct sockaddr_un serveraddr;

	  /********************************************************************/
	  /* The socket() function returns a socket descriptor representing   */
	  /* an endpoint.  The statement also identifies that the UNIX        */
	  /* address family with the stream transport (SOCK_STREAM) will be   */
	  /* used for this socket.                                            */
	  /********************************************************************/
	  sd = socket(AF_UNIX, SOCK_STREAM, 0);
	  if (sd < 0)
	    {
	      perror("socket() failed");
	      return;
	    }

	  /********************************************************************/
	  /* After the socket descriptor is created, a bind() function gets a */
	  /* unique name for the socket.                                      */
	  /********************************************************************/
	  memset(&serveraddr, 0, sizeof(serveraddr));
	  serveraddr.sun_family = AF_UNIX;
	  strcpy(serveraddr.sun_path, SERVER_PATH);

	  rc = bind(sd, (struct sockaddr *)&serveraddr, SUN_LEN(&serveraddr));
	  if (rc < 0)
	    {
	      perror("bind() failed");
	      return;
	    }

	  /********************************************************************/
	  /* The listen() function allows the server to accept incoming       */
	  /* client connections.  In this example, the backlog is set to 10.  */
	  /* This means that the system will queue 10 incoming connection     */
	  /* requests before the system starts rejecting the incoming         */
	  /* requests.                                                        */
	  /********************************************************************/
	  rc = listen(sd, 1);
	  if (rc< 0)
	    {
	      perror("listen() failed");
	      return;
	    }
// 	  cout << "Ready for client connect()." << endl;

	  /********************************************************************/
	  /* The server uses the accept() function to accept an incoming      */
	  /* connection request.  The accept() call will block indefinitely   */
	  /* waiting for the incoming connection to arrive.                   */
	  /********************************************************************/
	  socket_fd = accept(sd, NULL, NULL);
	  if (socket_fd < 0)
	    {
	      perror("accept() failed");
	      return;
	    }
	  struct protoent *p;
	  int one=1;
	  p = getprotobyname("tcp");
	  setsockopt(socket_fd, p->p_proto, TCP_NODELAY, (const char*)&one, sizeof(one));


	}
      // AF_INET
      else
	{
// 	  cout << "Creating INET socket ..." << endl;

	  int tmp_socket_fd, clilen, ret;
	  struct sockaddr_in cli_addr, serv_addr;
	  memset((char *)&serv_addr, 0, sizeof(serv_addr));

	  tmp_socket_fd = socket(AF_INET,SOCK_STREAM,0);

	  serv_addr.sin_family = AF_INET;
	  serv_addr.sin_addr.s_addr = htonl(INADDR_ANY);
	  serv_addr.sin_port = htons(this->serv_tcp_port);

	  ret = bind(tmp_socket_fd, (struct sockaddr *)&serv_addr, sizeof(serv_addr));
	  if(ret==-1)
	    {
	      cerr << "Problem when binding socket ... closing the simulation  (CD_SOCKET)" << endl;
	      exit(EXIT_FAILURE);
	    }

// 	  cout << "Waiting for connection ..." << endl;
	  ret = listen(tmp_socket_fd, 1);
	  if(ret==-1)
	    {
	      cerr << "Problem when listening socket ... closing the simulation (CD_SOCKET)" << endl;
	      exit(EXIT_FAILURE);
	    }
	  clilen = sizeof(cli_addr);
	  socket_fd = accept(tmp_socket_fd, (struct sockaddr *) &cli_addr, (socklen_t *)&clilen);
	  closesocket(tmp_socket_fd);          // close original socket
	  if(socket_fd >= 0)
	    {
	      struct protoent *p;
	      int one=1;
	      p = getprotobyname("tcp");
	      setsockopt(socket_fd, p->p_proto, TCP_NODELAY, (const char*)&one, sizeof(one));
// 	      cout << "Connection accepted" << endl;
	    }
	  else
	    {
	      cerr << "Error (CD_SOCKET): couldn't accept external connection" << endl;
	      exit(EXIT_FAILURE);
	    }
	}
    }
}



int Cd_Socket::receiveBuffer(void* buffer, int buffer_size)
{
  ret_status=0;
  for(int nbytes=0;nbytes<buffer_size;)
    {
      /*** to be sure tha everything is received ***/
      ret_status=recv(socket_fd,((char*)buffer)+nbytes,buffer_size-nbytes,0);
      if(ret_status > 0)
	nbytes+=ret_status;
      else
        {
	  break;
        }
    }
  return(ret_status);
}

int Cd_Socket::sendBuffer(void* buffer, int buffer_size)
{
  return int(send(socket_fd,buffer,buffer_size,0));
}

//) Cd_Socket


//) cd_socket.cpp

